import uuid
import requests
import urllib.parse
from .File import File
from .Preset import Preset
from .SupplyChain import SupplyChain

class Asset:

    def __init__(self,Rally,id=None,name=None,copy=True):
        self.Rally = Rally
        self.id = id
        self.name = name
        if not id and not name:
            self.name = "Temp_Asset_{}".format(str(uuid.uuid4()))
            self.id = Rally.apiCall("POST","/assets",body={"data":{"attributes":{"name":self.name},"type":"assets"}})["data"]["id"]
        elif not copy:
            if not id:
                self.name = name
                try:
                    self.id = Rally.apiCall("GET","/assets?search=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
                except IndexError:
                    raise ValueError("Could not find asset with name '{}'".format(name)) from None
        else:
            if not id:
                assetName = name
                try:
                    self.id = Rally.apiCall("GET","/assets?search=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
                except IndexError:
                    raise ValueError("Could not find asset with name '{}'".format(name)) from None
            elif not name:
                assetName = Rally.apiCall("GET","/assets/{}".format(self.id))["data"]["attributes"]["name"]
            self.name = "{}_Temp_Asset_{}".format(assetName,str(uuid.uuid4()))
            payload = Rally.apiCall("GET","/assets/{}".format(self.id))
            payload["data"]["attributes"] = {"name":self.name,"tagList":payload["data"]["attributes"]["tagList"],"status":payload["data"]["attributes"]["status"]}
            newId = Rally.apiCall("POST","/assets",body=payload)["data"]["id"]
            fileData = self.Rally.apiCall("GET","/assets/{}/files".format(self.id),paginate=True)["data"]
            for file in fileData:
                for instance in file["attributes"]["instances"]:
                    if file["attributes"]["instances"][instance]["storageLocationName"] == "Rally Platform Bucket":
                        del file["attributes"]["instances"][instance]
                file["relationships"]["asset"] = {"data":{"id":newId,"type":"assets"}}
                self.Rally.apiCall("POST","/files",body={"data":file})
            oldId = self.id
            self.id = newId
            try:
                metadataPayload = Rally.apiCall("GET","/userMetadata/{}".format(oldId))
                Rally.apiCall("PATCH","/userMetadata/{}".format(self.id),body=metadataPayload)
            except:
                pass
            wfmPresetName = "Add Workflow Metadata"
            try:
                workflowMetadata = Rally.apiCall("GET","/supplyChainMetadata/{}".format(oldId))["data"]["attributes"]["metadata"]
                try:
                    workflowMetadataPresetId = Rally.apiCall("GET","/presets?filter=name={}".format(urllib.parse.quote_plus(wfmPresetName)))["data"][0]["id"]
                    workflowMetadataPreset = Preset(self,id=workflowMetadataPresetId)
                    workflowMetadataPreset.run(dynamicPresetData=workflowMetadata,timeout=10)
                except IndexError:
                    providerTypeId = Rally.apiCall("GET","/providerTypes?filter=name=SdviEvaluate")["data"][0]["id"]
                    workflowMetadataPresetId = Rally.apiCall("POST","/presets",body={"data": {"type": "presets","attributes": {"name": wfmPresetName},"relationships": {"providerType": {"data": {"id": providerTypeId,"type": "providerTypes"}}}}})["data"]["id"]
                    Rally.apiCall("PUT","/presets/{}/providerData".format(workflowMetadataPresetId),body="WORKFLOW_METADATA = {{DYNAMIC_PRESET_DATA}}",fullResponse=True)
                    workflowMetadataPreset = Preset(self,id=workflowMetadataPresetId)
                    workflowMetadataPreset.run(dynamicPresetData=workflowMetadata,timeout=10)
            except:
                pass

    def getName(self):
        if not self.name:
            self.name = self.Rally.apiCall("GET","/assets/{}".format(self.id))["data"]["attributes"]["name"]
        return self.name

    def getMetadata(self):
        self.metadata = self.Rally.apiCall("GET","/movies/{}/metadata/Metadata".format(self.id))["data"]["attributes"]["metadata"]
        return self.metadata

    def getWorkflowMetadata(self):
        self.workflowMetadata = self.Rally.apiCall("GET","/movies/{}/metadata/Workflow".format(self.id))["data"]["attributes"]["metadata"]
        return self.workflowMetadata
    
    def delete(self):
        try:
            self.deleteMocks()
        except:
            pass
        if not any([silo in self.Rally.apiUrl for silo in ["dev","qa","uat"]]):
            raise ValueError("Production assets are read only") from None
        return self.Rally.apiCall("DELETE","/assets/{}".format(self.id),fullResponse=True)

    def deleteMocks(self):
        metadata = self.getMetadata()
        for imposter in metadata.get("mountebankImposters",[]):
            baseUrl = metadata["mountebankImposters"][imposter]
            port = baseUrl.split("/")[-1]
            requests.delete(url="http://rally-mountebank.{}.dcitech.cloud/2525/imposters/{}".format(self.Rally.env.lower(),port))

    def preset(self,id=None,name=None):
        return Preset(self,id=id,name=name)

    def supplychain(self,id=None,name=None):
        return SupplyChain(self,id=id,name=name)

    def listFiles(self):
        fileData = self.Rally.apiCall("GET","/assets/{}/files".format(self.id),paginate=True)["data"]
        return [File(self,id=item["id"],label=item["attributes"]["label"]) for item in fileData]

    def getFile(self,id=None,label=None):
        return File(self,id=id,label=label)

    def revertMovieFile(self):
        if not any([silo in self.Rally.apiUrl for silo in ["dev","qa","uat"]]):
            raise ValueError("Production assets are read only") from None
        originalFileId = self.getFile(label="OriginalSdviMovieFile").id
        currentFileId = self.getFile(label="SdviMovieFile").id
        self.Rally.apiCall("DELETE","/files/{}?mode=forget".format(currentFileId),fullResponse=True)
        originalFile = self.Rally.apiCall("GET","/files/{}".format(originalFileId))
        originalFile["data"]["attributes"]["label"] = "SdviMovieFile"
        self.Rally.apiCall("POST","/files",body=originalFile)
        self.Rally.apiCall("DELETE","/files/{}?mode=forget".format(originalFileId),fullResponse=True)


import ast
import json

class Job:

    def __init__(self,Preset,id=None,attributes=None):
        if not id:
            raise TypeError("Please specify the job id")
        self.id = id
        self.Rally = Preset.Asset.Rally
        self.Asset = Preset.Asset
        self.Preset = Preset
        self.presetName = Preset.name
        if not attributes:
            attributes = self.Rally.apiCall("GET","/jobs/{}".format(self.id))["data"]["attributes"]
        self.result = attributes["result"]
        self.dynamicPresetData = attributes["dynamicPresetData"]
        
    def getArtifact(self,name,parse=False):
        artifact = self.Rally.apiCall("GET","/artifacts/{}_{}/content".format(self.id,name),fullResponse=True).text
        if parse:
            oneLine = artifact.replace("'\n '","").replace("\n('","").replace("')\n","").replace('"\n "','').replace('"\n("','').replace('")\n','').replace("\n","")
            if "[" not in oneLine or (oneLine.find("{") < oneLine.find("[") and "{" in oneLine):
                curlyParse = oneLine[oneLine.find("{"):oneLine.rfind("}")+1]
                try:
                    return json.loads(curlyParse)
                except:
                    try:
                        return ast.literal_eval(curlyParse)
                    except:
                        raise ValueError("Could not parse output to a dictionary")
            else:
                squareParse = oneLine[oneLine.find("["):oneLine.rfind("]")+1]
                try:
                    return json.loads(squareParse)
                except:
                    try:
                        return ast.literal_eval(squareParse)
                    except:
                        raise ValueError("Could not parse output to a dictionary")
        return artifact


from .Job import Job
from time import sleep,time
import urllib.parse
import json
import redis
from .functions import *
class SupplyChain:

    def jsonPath(obj,pathString):
        pathParts = pathString.split(".")
        for pathPart in pathParts:
            obj = obj.get(pathPart,{}) or {}
        return obj

    def __init__(self,Asset,id=None,name=None):
        if not id and not name:
            raise TypeError("Please specify either the name or id of the rule that starts the supply chain")
        self.Rally = Asset.Rally
        self.Asset = Asset
        self.id = id
        self.name = name
        if not id:
            try:
                self.id = self.Rally.apiCall("GET","/workflowRules?filter=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
            except IndexError:
                raise ValueError("Could not find rule with name '{}'".format(name)) from None

    def run(self,initData={},endingPresetName=None,endingPresetId=None,timeout=15,protectProd=True):
        if not any([silo in self.Rally.apiUrl for silo in ["dev","qa","uat"]]) and protectProd:
            raise ValueError("Production assets are read only") from None
        if endingPresetName and not endingPresetId:
            endingPresetId = self.Asset.preset(name=endingPresetName).id
        self.Rally.apiCall("PATCH","/userMetadata/{}".format(self.Asset.id),body={"data":{"attributes":{"metadata":{"testMode":True}},"type":"userMetadata"}})
        baseWorkflow = self.Rally.apiCall("POST","/workflows",body={"data":{"type":"workflows", "attributes":{"initData":json.dumps(initData)}, "relationships": {"movie":{"data":{"id": self.Asset.id,"type": "movies",}},"workflowRule":{"data": {"id": self.id, "type": "workflowRules",}}}}})["data"]
        jobs = []
        workflows = [baseWorkflow]
        if self.Rally.redisUrl:
            redisClient = redis.Redis.from_url(url=self.Rally.redisUrl,
                ssl_cert_reqs=None,
                retry_on_timeout=True,
                socket_keepalive=True)
            subscriber = redisClient.pubsub()
            subscriber.subscribe('messagebus')
            start_time = time()
            delay = 0.1
            while (time()-start_time < timeout):
                response = subscriber.get_message()
                if response and response["data"] != 1:
                    data = json.loads(response["data"])
                    attributes = jsonPath(data,"resourceState.data.attributes") or {}
                    relationships = jsonPath(data,"resourceState.data.relationships") or {}
                    result = attributes.get("result")
                    if data["resourceType"] == "workflows":
                        assetId = jsonPath(relationships,"baseWorkflow.data.id")
                        baseWorkflowId = jsonPath(relationships,"asset.data.id")
                        if baseWorkflow["id"] == data["resourceId"] and result != None:
                            break
                        elif assetId == self.Asset.id and baseWorkflowId == baseWorkflow["id"]:
                            workflows.append(jsonPath(data,"resourceState.data"))
                            workflows = list({v['id']:v for v in workflows}.values())
                    elif data["resourceType"] == "jobs":
                        workflowId = jsonPath(relationships,"workflow.data.id")
                        if workflowId in [i["id"] for i in workflows] and result != None:
                            if endingPresetId == jsonPath(relationships,"preset.data.id"):
                                break
        else:
            delay = 5
            start_time = time()
            while (time()-start_time < timeout):
                sleep(delay)
                baseWorkflow = self.Rally.apiCall("GET","/workflows/{}".format(baseWorkflow["id"]))["data"]
                workflows = [baseWorkflow]
                childWorkflows = [item for item in self.Rally.apiCall("GET","/workflows?filter=assetId={}".format(self.Asset.id),paginate=True)["data"] if item["relationships"]["baseWorkflow"]["data"]["id"] == baseWorkflow["id"] and item["id"] != baseWorkflow["id"]]
                workflows.extend(childWorkflows)
                if baseWorkflow["attributes"]["result"] != None:
                    break
                if endingPresetId:
                    for workflow in workflows:
                        endingPresetJob = self.Rally.apiCall("GET","/jobs?filter=workflowId={},presetId={}".format(workflow["id"],endingPresetId),paginate=True)["data"]
                        if len(endingPresetJob) != 0:
                            if endingPresetJob[0]["attributes"]["result"] != None:
                                break
                    else:
                        continue
                    break
        for workflow in workflows:
            workflow_jobs = [i["id"] for i in self.Rally.apiCall("GET","/workflows/{}".format(workflow["id"]))["data"]["relationships"]["jobs"]["data"]]
            for workflow_job in workflow_jobs:
                job_data = self.Rally.apiCall("GET","/jobs/{}".format(workflow_job))["data"]
                if job_data["attributes"]["result"] != None:
                    jobs.append(Job(self,id=job_data["id"],attributes=job_data["attributes"]))     
                else:
                    self.Rally.apiCall("PATCH","/jobs/{}".format(job_data["id"]),body={"data":{"type":"jobs", "attributes":{"state":"Cancelled"}}},fullResponse=True,errors=False)
        self.Rally.apiCall("PATCH","/userMetadata/{}".format(self.Asset.id),body={"data":{"attributes":{"metadata":{"testMode":None}},"type":"userMetadata"}})
        return jobs


from colorama import Fore,Style,init
import os

if not os.environ.get("forceColor") == "1":
    init()

class Test:

    def __init__(self):
        self.tests = {}
    
    def test(self,condition,testName,**kwargs):
        self.tests[testName] = {
            "result":"pass" if condition else "fail",
            "kwargs": kwargs
        }
    
    def print(self):
        print("---------------")
        for testName in self.tests:
            test = self.tests[testName]
            print("\033[1m" + "Name: " + Style.RESET_ALL + Fore.CYAN + testName + Style.RESET_ALL)
            print("\033[1m" + "Result: " + Style.RESET_ALL + (Fore.GREEN if test["result"] == "pass" else Fore.RED) + test["result"] + Style.RESET_ALL)
            for key in test["kwargs"]:
                print(Style.DIM + str(key) + " = " + str(test["kwargs"][key]))
            print(Style.RESET_ALL+"---------------")
        if any([self.tests[testName]["result"] == "fail" for testName in self.tests]):
            os.environ["testsFailed"] = "1"


import requests
import urllib.parse

class File:

    def __init__(self,Asset,id=None,label=None):
        self.id = id
        self.Rally = Asset.Rally
        self.Asset = Asset
        self.label = label
        if not label and not id:
            raise TypeError("Please specify either a label or id")
        elif not id:
            try:
                self.id = self.Rally.apiCall("GET","/assets/{}/files?filter=label={}".format(self.Asset.id,urllib.parse.quote_plus(self.label)))["data"][0]["id"]
            except IndexError:
                raise ValueError("Could not find file with label '{}'".format(label)) from None

    def getLabel(self):
        if not self.label:
            self.label = self.Rally.apiCall("GET","/files/{}".format(self.id))["data"]["attributes"]["label"]
        return self.label

    def getAnalyzeInfo(self):
        mediaAttributesLinks = self.Rally.apiCall("GET","/files/{}/mediaAttributes?no-redirect=true".format(self.id))["links"]
        self.analyzeInfo = requests.get(mediaAttributesLinks["mediaAttributes"]).json()
        return self.analyzeInfo


from rallytt.Rally import Rally
from rallytt.Asset import Asset
from rallytt.File import File
from rallytt.Preset import Preset
from rallytt.SupplyChain import SupplyChain
from rallytt.Job import Job
from rallytt.Test import Test


def jsonPath(obj,pathString):
    pathParts = pathString.split(".")
    for index,pathPart in enumerate(pathParts):
        if index != (len(pathParts) - 1):
            obj = obj.get(pathPart,{}) or {}
        else:
            obj = obj.get(pathPart)
    return obj


from setuptools import setup, find_packages

VERSION = '0.3.7' 
DESCRIPTION = 'Integration/E2E test tools for SDVI Rally'
LONG_DESCRIPTION = 'A set of tools for running and verifying results for presets/supply chains in SDVI Rally'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="rallytt", 
        version=VERSION,
        author="Carson O'Ffill",
        author_email="offillcarson@gmail.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=["requests","colorama","redis"],
        keywords=['python', 'sdvi rally', 'integration test', 'end to end test', 'e2e test', 'testing tools'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows"
        ]
)


from time import sleep,time
import urllib.parse
import redis
import json
from .Job import Job

class Preset:

    def __init__(self,Asset,id=None,name=None):
        if not id and not name:
            raise TypeError("Please specify either the name or id of the preset")
        self.Rally = Asset.Rally
        self.Asset = Asset
        self.id = id
        self.name = name
        if not id:
            try:
                self.id = self.Rally.apiCall("GET","/presets?filter=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
            except IndexError:
                raise ValueError("Could not find preset with name '{}'".format(name)) from None

    def getName(self):
        self.name = self.Rally.apiCall("GET","/presets/{}".format(self.id))["data"]["attributes"]["name"]
        return self.name

    def run(self,dynamicPresetData={},timeout=5,protectProd=True):
        if not any([silo in self.Rally.apiUrl for silo in ["dev","qa","uat"]]) and protectProd:
            raise ValueError("Production assets are read only") from None
        self.Rally.apiCall("PATCH","/userMetadata/{}".format(self.Asset.id),body={"data":{"attributes":{"metadata":{"testMode":True}},"type":"userMetadata"}})
        jobId = self.Rally.apiCall("POST","/jobs",body={"data":{"type":"jobs", "attributes":{"dynamicPresetData":dynamicPresetData}, "relationships": {"movie":{"data":{"id": self.Asset.id,"type": "movies",}},"preset":{"data": {"id": self.id,"type": "presets",}}}}})["data"]["id"]
        if self.Rally.redisUrl:
            redisClient = redis.Redis.from_url(url=self.Rally.redisUrl,ssl_cert_reqs=None)
            subscriber = redisClient.pubsub()
            subscriber.subscribe('messagebus')
            start_time = time()
            delay = 0.1
            while (time()-start_time < timeout):
                response = subscriber.get_message()
                if response and response["data"] != 1:
                    data = json.loads(response["data"])
                    result = data.get("resourceState",{}).get("data",{}).get("attributes",{}).get("result")
                    if jobId == data["resourceId"] and result != None:
                        return Job(Preset=self,id=jobId,attributes=data.get("resourceState",{}).get("data",{}).get("attributes",{}))
                sleep(delay)
        else:
            start_time = time()
            delay = 2
            while (time()-start_time < timeout):
                sleep(delay)
                job = self.Rally.apiCall("GET","/jobs/{}".format(jobId))["data"]
                if job["attributes"]["state"] not in ["Queued","Active"]:
                    self.Rally.apiCall("PATCH","/userMetadata/{}".format(self.Asset.id),body={"data":{"attributes":{"metadata":{"testMode":None}},"type":"userMetadata"}})
                    return Job(Preset=self,id=jobId,attributes=job["attributes"])

    def job(self,id=None,attributes=None):
        return Job(self,id=id,attributes=attributes)


import os
import requests
import json
from .Asset import Asset
import re

class Rally:

    def __init__(self,findParams=True,env="UAT",apiUrl=None,apiKey=None,redisUrl=None):
        self.apiUrl = apiUrl
        self.apiKey = apiKey
        self.redisUrl = redisUrl
        if findParams:
            if os.environ.get("apiKey") and os.environ.get("apiUrl"):
                self.apiUrl = os.environ.get("apiUrl")
                self.apiKey = os.environ.get("apiKey")
                self.redisUrl = os.environ.get("redisUrl")
            else:
                configPath = os.path.expanduser('~')+"/.rallyconfig"
                rallyConfig = open(configPath)
                rallyConfigJson = json.load(rallyConfig)
                apiConfig = rallyConfigJson["api"]
                rallyConfig.close()
                self.apiUrl = apiConfig[env]["url"]
                self.apiKey = apiConfig[env]["key"]
                self.redisUrl = "rediss://127.0.0.1:{}".format(rallyConfigJson.get("redis",{}).get(env)) if rallyConfigJson.get("redis") else None
        elif not apiUrl or not apiKey:
            raise TypeError("Please specify both apiUrl and apiKey parameters")
        matches = re.findall("discovery.*sdvi", self.apiUrl)
        if len(matches) == 0:
            raise ValueError("Cannot extract environment from api url")
        env = matches[0].replace(".sdvi","").replace("discovery","")
        self.env = "PROD" if env == "" else env.replace("-","").upper()
    
    def apiCall(self,method,endpoint,body={},paginate=False,fullResponse=False,errors=True):
        headers={"Authorization":"Bearer {}".format(self.apiKey),"Content-Type":"application/json" if type(body) is dict else "text/plain"}
        response = requests.request(method,headers=headers,url="{}{}".format(self.apiUrl,endpoint),data=json.dumps(body) if type(body) is dict else body)
        if errors:
            response.raise_for_status()
        if not fullResponse or paginate:
            response = response.json()
        page = 2
        while paginate:
            if "?" in endpoint:
                url = "{}{}&page={}p10".format(self.apiUrl,endpoint,page)
            else:
                url = "{}{}?page={}p10".format(self.apiUrl,endpoint,page)
            results = requests.request(method,headers=headers,url=url,data=json.dumps(body))
            if results.status_code != 404:
                if errors:
                    results.raise_for_status()
                response["data"].extend(results.json()["data"])
                if len(results.json()["data"]) < 10:
                    paginate = False
                page+=1
            else:
                paginate = False
        return response

    def asset(self,id=None,name=None,copy=True):
        return Asset(self,id=id,name=name,copy=copy)